import { AxiosInstance, RawAxiosRequestHeaders, AxiosRequestConfig } from 'axios';
import { HTTP_METHODS } from '../enums';
export declare enum JsonApiLinkFields {
    first = "first",
    last = "last",
    next = "next",
    prev = "prev",
    self = "self"
}
export declare type URL = string;
export declare type Endpoint = string;
export declare type JsonApiClientConfig = {
    baseUrl?: URL;
    axios?: AxiosInstance;
};
export declare type JsonApiClientRequestConfigHeaders = RawAxiosRequestHeaders;
export declare type JsonApiClientRequestParams = unknown;
export declare type JsonApiClientRequestConfig = AxiosRequestConfig;
export declare type JsonApiErrorMetaType = Record<string, unknown> | unknown[] | unknown;
export declare type JsonApiRelationship = Record<string, unknown>;
export declare type JsonApiRelationships = Record<string, JsonApiRelationship | JsonApiRelationship[]>;
export declare type JsonApiRecordBase<T extends string> = {
    id: string;
    type: T;
    relationships?: JsonApiRelationships;
};
export declare type JsonApiResponseLinks = {
    first?: Endpoint;
    last?: Endpoint;
    next?: Endpoint;
    prev?: Endpoint;
    self?: Endpoint;
};
export declare type JsonApiClientRequestOpts = {
    endpoint: Endpoint;
    method: HTTP_METHODS;
    headers?: JsonApiClientRequestConfigHeaders;
    data?: unknown;
    query?: unknown;
    contentType?: string;
    isEmptyBodyAllowed?: boolean;
    isNeedRaw?: boolean;
    withCredentials?: boolean;
};
export declare type JsonApiResponseError = {
    id?: string | number;
    code?: string;
    title?: string;
    detail?: string;
    status?: string;
    source?: {
        pointer?: string;
        parameter?: string;
        header?: string;
    };
    meta?: JsonApiErrorMetaType;
    links?: JsonApiResponseLinks;
};
export declare type JsonApiResponseNestedErrors = JsonApiResponseError[];
export declare type JsonApiResponseErrors = {
    errors?: JsonApiResponseNestedErrors;
};
